
#include "boards.h"
#include "app_util_platform.h"
#include "bsp.h"
#include "app_error.h"
#include "app_twi.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"

#define TAMANY_BUF_TX 50
#define TAMANY_CUA 5
#define INTENTS_MAXIMS 5
#define DEV_TYPE_ID_USER_MEMORY_R 0xA7
#define DEV_TYPE_ID_USER_MEMORY_W 0xA6
#define DEV_TYPE_ID_SYSTEM_MEMORY_R 0xAF
#define DEV_TYPE_ID_SYSTEM_MEMORY_W 0xAE
#define BYTE_WRITE_LEN 3
#define PAGE_WRITE_LEN 6
#define CONFIG_BYTE_ADDR_MSB 0x09
#define CONFIG_BYTE_ADDR_LSB 0x10
#define CONFIG_BYTE_DATA_INDEX 2
#define P_MIN_100MILIW 0
#define P_MIN_60MILIW 1
#define P_MIN_30MILIW 2
#define P_MIN_16MILIW 3

static const nrf_drv_twi_t twi_master = NRF_DRV_TWI_INSTANCE(0);
static uint8_t configuration_byte[] = {CONFIG_BYTE_ADDR_MSB, CONFIG_BYTE_ADDR_LSB, 0x00};
static volatile bool transferencia_tx_feta;
static volatile bool transferencia_rx_feta;

/*
		*gestor d'events del twi, de la llibreria nrf_drv_twi de Nordic Semiconductor
*/
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
	switch (p_event->type)
	{
		case NRF_DRV_TWI_EVT_DONE: //indica que una transferencia s'ha fet amb exit
			if(p_event->xfer_desc.type == NRF_DRV_TWI_XFER_TX)
			{
				transferencia_tx_feta = true;
			}
			else if(p_event->xfer_desc.type == NRF_DRV_TWI_XFER_RX)
			{
				transferencia_rx_feta = true;
			}
			break;
		case NRF_DRV_TWI_EVT_ADDRESS_NACK: //no s'ha rebut el acknowledge bit despres d'enviar l'adrea
			break;
		case NRF_DRV_TWI_EVT_DATA_NACK: //no s'ha rebut el acknowledge bit despres d'enviar les dades
			break;
	}
}

/*
   *configurar_twi: Funcio que configura el driver TWI del nrf52832 amb els
										parametres que s'indiquen al fitxer "nrf_drv_config.h" del
										twi que hem decidit fer servir. Tambe inicialitza el twi, 
										la isntancia de twi que hem creat amb APP_TWI_INSTANCE, ha
										de ser la mateixa que el twi que hem activat a "nrf_drv_config.h"
	*no te parametres
	*no retorna res
*/
uint32_t configurar_twi(void)
{
		uint32_t error;
	
		nrf_drv_twi_config_t const configuracio = {
			.scl = TWI0_CONFIG_SCL, //27
			.sda = TWI0_CONFIG_SDA, //26
			.frequency = TWI0_CONFIG_FREQUENCY, //NRF_TWI_FREQ_100K
			.interrupt_priority = TWI0_CONFIG_IRQ_PRIORITY //APP_IRQ_PRIORITY_LOW
		};
		
		error = nrf_drv_twi_init(&twi_master, &configuracio, twi_handler, NULL);
		nrf_drv_twi_enable(&twi_master);
		return error;
}

/*
   *valor_configuration_byte: Funcio que estableix el valor que dessitjem escriure
										al registre de sistema "configuration byte", que sera el que
										activi o no, el us del pin RF WIP/BUSY, el energy harvesting,
										i la potencia minima que ha d'arribar a l'antena per a que, en
										cas de ser activat, transmeti l'energia sobrant pel pin V OUT

	*rfwip_busy: 			si es true activa l'us del pin RF WIP/BUSY
	*eh_mode:					si es false, quan es torni a encendre el M24LR04E, l'energy
										harvesting estara activat
	*eh_cfg:					minim valor de potencia rebuda per activar l'energy harvesting
										nomes pot valdre 0, 1, 2 o 3
	*config_byte			byte on es vol guardar el valor final de "configuration byte"

	*return						retorna NRF_ERROR_INVALID_PARAM si el parametre eh_cfg te un
										valor incorrecte
*/
uint32_t valor_configuration_byte(bool rfwip_busy, bool eh_mode, uint8_t eh_cfg)
{
		uint32_t error;

		configuration_byte[CONFIG_BYTE_DATA_INDEX] = 0xA0; //4 primers bits no importen 
		if(rfwip_busy) configuration_byte[CONFIG_BYTE_DATA_INDEX] ^= 0x08;
		if(eh_mode) configuration_byte[CONFIG_BYTE_DATA_INDEX] ^= 0x04;
		switch (eh_cfg)
		{
			case 0:
				error = NRF_SUCCESS;
			break;
			case 1:
				configuration_byte[CONFIG_BYTE_DATA_INDEX] ^= 0x01;
				error = NRF_SUCCESS;
			break;
			case 2:
				configuration_byte[CONFIG_BYTE_DATA_INDEX] ^= 0x02;
				error = NRF_SUCCESS;
			break;
			case 3:
				configuration_byte[CONFIG_BYTE_DATA_INDEX] ^= 0x03;
				error = NRF_SUCCESS;
			break;
			default: //valor de eh_cfg incorrecte
				error = NRF_ERROR_INVALID_PARAM;
			break;
		}
		return error;
}

int main(void)
{
		uint32_t error;
//		nrf_drv_twi_xfer_desc_t const escriure_configuration_byte ={ //transferencia que es fa quan vol escriure el configuration byte
//			.type = NRF_DRV_TWI_XFER_TX, //volem una transferencia que envii dades nomes
//			.address = DEV_TYPE_ID_SYSTEM_MEMORY_W, //el byte inclou que es vol accedir a la memoria del sistema per escriure
//			.primary_length = BYTE_WRITE_LEN,
//			.secondary_length = NULL,
//			.p_primary_buf = configuration_byte, //array de 3 bytes: adrea MSB + adrea LSB + configuration byte
//			.p_secondary_buf = NULL
//		};
		
		error = configurar_twi();
		error = valor_configuration_byte(true, false, P_MIN_60MILIW); //modyfies the value of the 3rd byte of configuration_byte
		if(error == NRF_SUCCESS)
		{
			transferencia_tx_feta = false;
			error = nrf_drv_twi_tx(&twi_master, DEV_TYPE_ID_SYSTEM_MEMORY_W, configuration_byte, sizeof(configuration_byte), false);
//			error = nrf_drv_twi_xfer(&twi_master, &escriure_configuration_byte, NULL);
			do
			{
				__WFE;
			}while(transferencia_tx_feta == false);
		}
		
}
















































